module plot_ascii

   implicit none

   character, parameter :: esc = ACHAR(27)

contains

   subroutine csc

! Erase display, put cursor in upper left corner
      write(*,fmt='(a1,a)',advance='no') esc,'[0m' ! All attributes off
      write(*,fmt='(a1,a)',advance='no') esc,'[2J'
      write(*,fmt='(a1,a)',advance='no') esc,'[H' 

   end subroutine csc

   subroutine sgr( n )

      integer, optional                         :: n

! Select graphic rendition

      if ( .not. present(n) ) then
         write(*,fmt='(a1,a)',advance='no') esc,'[0m'
      else
         write(*,fmt='(a1,a1,i3,a1)',advance='no') esc,'[',n,'m' 
      end if

   end subroutine sgr

   subroutine cup( ix, iy )

      integer                                   :: ix, iy

! Set cursor position
      write(*,fmt='(a1,a1,i3,a1,i3,a1)',advance='no') esc,'[',iy,';',ix,'H'

   end subroutine cup

   subroutine cuu( n )

      integer                                   :: n

! Cursor position n positions up
      write(*,fmt='(a1,a1,i3,a1)',advance='no') esc,'[',n,'A'

   end subroutine cuu

   subroutine cud( n )

      integer                                   :: n

! Cursor position n positions down
      write(*,fmt='(a1,a1,i3,a1)',advance='no') esc,'[',n,'B'

   end subroutine cud

   subroutine cuf( n )

      integer                                   :: n

! Cursor position n positions forward
      write(*,fmt='(a1,a1,i3,a1)',advance='no') esc,'[',n,'C'

   end subroutine cuf

   subroutine cub( n )

      integer                                   :: n

! Cursor position n positions back
      write(*,fmt='(a1,a1,i3,a1)',advance='no') esc,'[',n,'D'

   end subroutine cub

   subroutine scp

! Save cursor position
      write(*,fmt='(a1,a)',advance='no') esc,'[s'

   end subroutine scp

   subroutine rcp

! Restore cursor position
      write(*,fmt='(a1,a)',advance='no') esc,'[u'

   end subroutine rcp

   subroutine pixel( colour, ix, iy )

      character(len=12)                         :: colour
      integer, optional                         :: ix, iy

! Move cursor to position (ix,iy) (optional) 
      if ( present(ix) .and. present(iy) ) call cup( ix, iy )

! Paint pixel location with colour 
      write(*,fmt='(a)',advance='no') colour

   end subroutine pixel

   function palet( which_palet )

! Define pixel colours

      character(len=12), allocatable      :: palet(:)
      character(len=12), allocatable      :: colour(:)
      character(len=6)                    :: which_palet
      integer                             :: i

      if ( which_palet == 'bright' ) then
         allocate( colour(8), palet(8) )
         do i = 1, 8
            write(colour(i),fmt='(a1,a1,i8,a2)') esc,'[',i+99,'m '
         end do
      elseif ( which_palet == 'greysc' ) then
         allocate( colour(24), palet(24) )
         do i = 1, 24
            write(colour(i),fmt='(a1,a6,i3,a2)') esc,'[48;5;',i+231,'m '
         end do
      elseif ( which_palet == '216col' ) then
         allocate( colour(216), palet(216) )
         do i = 1, 216
            write(colour(i),fmt='(a1,a6,i3,a2)') esc,'[48;5;',i+15,'m '
         end do
      else
         allocate( colour(8), palet(8) )
         do i = 1, 8
            write(colour(i),fmt='(a1,a1,i8,a2)') esc,'[',i+39,'m '
         end do
      end if
      palet = colour

   end function palet

   subroutine image( f, legend, which_palet, cmin, cmax )

      integer                                    :: m, n
      character(len=12), allocatable             :: colour(:)
      character(len=80), optional, intent(in)    :: legend
      character(len=6), optional, intent(in)     :: which_palet
      real, optional                             :: cmin, cmax
      real                                       :: f(:,:), min_val, max_val
      integer                                    :: i,j, c(size(f,1),size(f,2)), ncol

      m = size(f,1)
      n = size(f,2)

      if ( present(which_palet) ) then
         colour = palet( which_palet )
      else
         colour = palet( 'normal' )
      end if
      ncol   = size(colour)

      call cud( n )

      if ( present(cmin) ) then
         min_val = cmin
      else
         min_val = minval(f)
      end if
      if ( present(cmax) ) then
         max_val = cmax
      else
         max_val = maxval(f)
      end if

      c = int(ncol*(f - min_val)/(max_val-min_val))+1

      do j = 1,n
         do i = 1,m
            if ( c(i,j) < 1 ) c(i,j) = 1
            if ( c(i,j) > ncol ) c(i,j) = ncol
         end do
       end do

! Plot the image
      do j = 1,n
         call cuu( 1 )
         do i = 1,m
            call pixel(colour(c(i,j)))
         end do
         call cub( m )
      end do

      call cud( n )
      call sgr                 ! All attributes off
      if ( present(legend) ) write(*,fmt='(a,/)') legend

   end subroutine image

end module plot_ascii
